package DZT::Sample;
# ABSTRACT: Sample DZ Dist

=encoding utf8

=head1 SYNOPSIS

  use DZT::Sample;

=cut

use strict;
use warnings;

our $VERSION = '0.04';

1;

=head1 DEPENDENCIES

DZT::Sample requires ümlauts.

=for Pod::Loom-omit
BUGS AND LIMITATIONS
CONFIGURATION AND ENVIRONMENT
COPYRIGHT AND LICENSE
DISCLAIMER OF WARRANTY
INCOMPATIBILITIES
